package cn.demo.sys;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.JSONPath;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.validation.ConstraintViolationException;


/**
 * 根据需要，此处可以全局处理
 * 各种异常应该返回的 code/msg信息
 */
@Slf4j
@RestControllerAdvice(annotations = RestController.class)
public class RestGlobalExceptionHandler {
    /**
     * 运行时业务异常
     *
     * @param e APIException
     * @return String
     */
    @ExceptionHandler({APIException.class})
    public ResultVO<String> APIExceptionHandler(APIException e) {
        // 注意哦，这里返回类型是自定义响应体
        return new ResultVO<>(e.getCode(), e.getMsg());
    }



    /**
     * @param e HttpMessageNotReadableException
     * @return ResultVO
     * @RequestBody @Valid put/post请求体缺少异常
     */
    @ExceptionHandler({HttpMessageNotReadableException.class})
    public ResultVO reqBodyIsNullException(HttpMessageNotReadableException e) {
        log.error("requestBody请求体异常",e);
        return new ResultVO(ResultCodeEnum.REQ_BODY_IS_NULL, e.getMessage());
    }


    /**
     * get请求/post请求的 requestParam的参数异常
     * @param e MissingServletRequestParameterException
     * @return ResultVO
     */
    @ExceptionHandler({MissingServletRequestParameterException.class})
    public ResultVO reqMissingServletRequestParameterException(MissingServletRequestParameterException e){
        log.error("queryString请求参数异常",e);
        return new ResultVO(ResultCodeEnum.REQ_QUERY_STRING_IS_NULL, e.getMessage());
    }


    /**
     * 对 jsr303 参数校验失败抛出的MethodArgumentNotValidException全局处理
     *
     * @param e MethodArgumentNotValidException
     * @return String
     */
    @ExceptionHandler({MethodArgumentNotValidException.class})
    public ResultVO<String> MethodArgumentNotValidExceptionHandler(MethodArgumentNotValidException e) {
        log.error("自动抛的MethodArgumentNotValidException",e);
        // 从异常对象中拿到ObjectError对象
        ObjectError objectError = e.getBindingResult().getAllErrors().get(0);
        log.error("objectError:{}", JSONObject.toJSONString(objectError, true));
        String field = String.valueOf(JSONPath.eval(objectError, "$.field"));
        log.error("field:{}", field);
        String objName = objectError.getObjectName();
        System.err.println(objName);
        // 然后提取错误提示信息进行返回
        return new ResultVO<>(ResultCodeEnum.VALIDATE_FAILED,
                objName + "." + field + objectError.getDefaultMessage());
    }


//    // UnauthorizedException
//    @ExceptionHandler({UnauthorizedException.class})
//    public ResultVO<String> ErrorHandler2(UnauthorizedException e) {
//        log.error("没有通过权限验证！", e);
//        return new ResultVO<>(ResultCodeEnum.ACCOUNT_DISABLED, e.getMessage());
//    }


//    /**
//     * 未登录异常
//     * @param e AuthorizationException
//     * @return ResultVO<String>
//     */
//    @ExceptionHandler({cn.dev33.satoken.exception.NotLoginException.class})
//    public ResultVO<String> ErrorHandler(cn.dev33.satoken.exception.NotLoginException e) {
//        log.error("没有通过权限验证！", e);
//        return new ResultVO<>(ResultCodeEnum.NOT_LOGIN, e.getMessage());
//    }


    /**
     * 处理 ArithmeticException 类的异常 （如除0异常）
     */
    @ExceptionHandler({ArithmeticException.class})
    public ResultVO arithmeticExceptionHandler(ArithmeticException e) {
        return new ResultVO(ResultCodeEnum.ARITHMETIC_FAILED, e.getMessage());
    }

    /**
     * 手动抛出的或 Assert断言的 非法参数异常
     *
     * @param e IllegalArgumentException
     * @return ResultVO
     */
    @ExceptionHandler({IllegalArgumentException.class})
    public ResultVO illegalArgumentExceptionHandler(IllegalArgumentException e) {
        log.error("人工抛出异常的IllegalArgumentException",e);
        return new ResultVO<>(ResultCodeEnum.IllEGAL_ARG_EXCEPTION,
                e.getMessage());
    }


    @ExceptionHandler({ArrayIndexOutOfBoundsException.class})
    public ResultVO arrayIndexOutOfBoundsExceptionHandler(ArrayIndexOutOfBoundsException e) {
        return new ResultVO(ResultCodeEnum.ARRAY_INDEX_OUT_OF_BOUNDS_FAILED, e.getMessage());
    }


//    public static String getThrowableStackInfo(Throwable e) {
//        ByteArrayOutputStream buf = new ByteArrayOutputStream();
//        e.printStackTrace(new java.io.PrintWriter(buf, true));
//        String msg = buf.toString();
//        try {
//            buf.close();
//        } catch (Exception t) {
//            return e.getMessage();
//        }
//        return msg;
//    }

//    @ResponseBody
//    @ExceptionHandler(value = ArithmeticException.class)
//    public String handleArithmetic(HttpServletRequest request, HttpServletResponse response, ArithmeticException e)
//            throws IOException {
//        log.info("divide error!");
//        return "divide 0: " + getThrowableStackInfo(e);
//    }

//    @ResponseBody
//    @ExceptionHandler(value = ArrayIndexOutOfBoundsException.class)
//    public String handleArrayIndexOutBounds(HttpServletRequest request, HttpServletResponse response,
//                                            ArrayIndexOutOfBoundsException e) throws IOException {
//        log.info("array index out error!");
//        return "aryIndexOutOfBounds: " + getThrowableStackInfo(e);
//    }


    /**
     * Validated全局自动校验,要@Validated加在Controller类名上,同时手动使用BindingResult
     */
    @ExceptionHandler({ConstraintViolationException.class})
    public ResultVO constraintViolationExceptionHandler(ConstraintViolationException e) {
        log.error("Validated全局自动校验出的异常",e);
        //校验框架会自动返回多条约束提示信息的拼接
        //格式如 test1234.req.company.name: 长度需要在4-30之间,test1234.req.account: 长度需要在2-30之间
        //(逗号拼接的字符串  接口方法名.参数名.子参数对象.子参数属性:描述)
        return new ResultVO<>(ResultCodeEnum.VALIDATE_FAILED,
                e.getMessage());
    }


    /**
     * 未知异常
     *
     * @param e
     * @return
     */
    @ExceptionHandler({Exception.class})
    public ResultVO otherExceptionHandle(Exception e) {
        log.error("otherExceptionHandle", e);
        return new ResultVO(ResultCodeEnum.ERROR, e.getMessage());
    }


}

